/*!
 * vue-coe-validator v0.3.8
 * (c) 2018-present Vinicius Azevedo <viniazvd@gmail.com>
 * Released under the MIT License.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function getContext() {
  var vm = this.$validator || this;
  var componentID = Object.keys(vm.context.components)[Object.keys(vm.context.components).length - 1];
  return vm.context.components[componentID];
}
function setContext() {
  var obj;

  this.$validator.context.components = Object.assign({}, this.$validator.context.components, ( obj = {}, obj[this._uid] = this, obj));
}

function validate(form, key, value) {
  var vm = getContext.call(this);
  var validations = vm.validations;
  var messages = vm.messages;
  var state = validations[form][key];
  state.isLoading = true;
  var hasCustomAsyncRule = state['customAsync'];
  var syncErrors = getSyncErrors(validations, messages, form, key, value);
  var isTouched = validations[form] && state && state.isTouched;
  state.errors = syncErrors;
  state.isLoading = hasCustomAsyncRule && !!hasCustomAsyncRule.length || false;
  state.isChanged = true;
  state.isTouched = true;
  state.isDirty = !!value || isTouched;
  state.isFilled = !!value;
  state.isValid = syncErrors.length <= 0;

  if (hasCustomAsyncRule && value) {
    getAsyncErrors(validations, messages, form, key, value).then(function (asyncErrors) {
      state.errors = syncErrors.concat( [asyncErrors]);
      state.isLoading = false;
      state.isValid = syncErrors.concat( [asyncErrors]).length <= 0;
    });
  } // scenario:
  // -when the field has asynchronous validation
  // -field has 1 character typed and user erases (field is empty)


  if (hasCustomAsyncRule && !value) {
    state.isLoading = false;
    state.isFilled = false;
  }
}

var validateField = validate;

function alpha (value, msg) {
  return !Array.isArray(value) && !/^\s*([0-9a-zA-Z]*)\s*$/.test(value) && (msg || 'Must only contain letters and numbers');
}

function alphabetic (value, msg) {
  return !Array.isArray(value) && !/^[a-zA-Z]*$/.test(value) && (msg || 'Must be a alphabetic value');
}

function custom (value, msg, validations, form, key) {
  var callbacks = validations[form][key]['custom'];
  var valid = callbacks.every(function (f) { return f(value); });
  return (!Array.isArray(callbacks) || !valid) && (msg || 'Campo inválido');
}

function customAsync (value, msg, validations, form, key) {
  var callbacks = validations[form][key]['customAsync'];
  var promisses = callbacks.map(function (f) { return f(value); });
  return Promise.all(promisses).then(function (promises) { return promises.every(function (promise) { return promise; }); }).then(function (valid) { return (!Array.isArray(callbacks) || !valid) && (msg || 'Campo inválido'); });
}

function numeric (value, msg) {
  return !Array.isArray(value) && !/^[0-9]*$/.test(value) && (msg || 'Must be a numeric value');
}

function pattern (value, msg, forms, form, key) {
  var pattern = forms[form][key].pattern;
  return value && !pattern.test(value) && (msg || 'Campo inválido');
}

function required (value, msg) {
  return !value && (msg || 'Field is required');
}



var VALIDATIONS = /*#__PURE__*/Object.freeze({
  alpha: alpha,
  alphabetic: alphabetic,
  custom: custom,
  customAsync: customAsync,
  numeric: numeric,
  pattern: pattern,
  required: required
});

function setMessages(source, newMessages) {
  for (var form of Object.values(source)) {
    for (var input of Object.values(form)) {
      for (var messageKey of Object.keys(input)) {
        if (newMessages[messageKey]) {
          input[messageKey] = newMessages[messageKey];
        }
      }
    }
  }
}
var defaultState = {
  isLoading: false,
  isChanged: false,
  isTouched: false,
  isDirty: false,
  isFilled: false,
  isValid: false,
  errors: []
};
function getData() {
  /* eslint-disable */
  var _this$$data = this.$data;
  var validations = _this$$data.validations; if ( validations === void 0 ) validations = {};
  var messages = _this$$data.messages; if ( messages === void 0 ) messages = {};
  var data = _objectWithoutPropertiesLoose(_this$$data, ["validations", "messages"]);
  /* eslint-enable */


  return data;
}
function watchValidate(formKey, input) {
  var this$1 = this;

  var unwatch = this.$watch(formKey.concat('.', input), function (value) {
    validateField.call(this$1, formKey, input, value);
  });
  this.$on('hook:beforeDestroy', unwatch);
}
function setProxy() {
  // issue!?
  // setProxy works...
  // however, it creates a deep proxy reference.
  var validationsProxy = new Proxy(this.validations, {
    deleteProperty: function deleteProperty(target, prop) {
      if (prop in target) {
        process.env.NODE_ENV === 'development' && console.warn("you can not remove validations property");
        return true;
      }
    }

  });
  this.validations = validationsProxy;
}

function forceValidation(form, element, key, value) {
  if ( key === void 0 ) key = element.name;
  if ( value === void 0 ) value = element.value;

  var vm = getContext.call(this);
  var isTouched = vm.validations[form] && vm.validations[form][key] && vm.validations[form][key].isTouched; // to prevent unnecessary checks

  if (vm.validations && !isTouched) {
    vm.validations[form][element.name].isTouched = true;
    validateField.call(this, form, key, value || '');
  }
}

function addTouchListener(formName, inputElement, addByDirective) {
  var this$1 = this;

  // register events only for those who have validation
  var hasValidation = this.validations[formName] && this.validations[formName][inputElement.name]; // check if addEventListener has already been set in the conditional below? worth it?

  if (hasValidation || addByDirective) {
    inputElement.addEventListener('blur', function () { return forceValidation.call(this$1, formName, inputElement); }, {
      once: true
    });
  }
}
function setListenersTouch() {
  var vm = getContext.call(this); // dynamically records listeners to activate touch inputs

  vm.$nextTick(function () {
    // TO-DO. issue: does not work in tests
    var NodeListForms = vm.$el.querySelectorAll('form[name]');

    if (NodeListForms.length) {
      NodeListForms.forEach(function (NodeForm) {
        var formName = NodeForm.getAttribute('name');
        Array.from(NodeForm).forEach(function (input) { return addTouchListener.call(vm, formName, input); });
      });
    } else {
      console.warn('follow the instructions in the documentation to correctly register the form');
    }
  });
}
var RULES = Object.keys(VALIDATIONS);

function isRule(rule, validations, form, key) {
  return validations[form] && validations[form][key] && validations[form][key][rule];
}

function getMessage(rule, messages, form, key) {
  return messages && messages[form] && messages[form][key] && messages[form][key][rule];
}

function getError(rule, validations, form, key, value, msg) {
  return VALIDATIONS[rule](value, msg, validations, form, key);
}

function getSyncErrors(validations, messages, form, key, value) {
  var errors = [];
  RULES.some(function (rule) {
    if (isRule(rule, validations, form, key) && rule !== 'customAsync') {
      var msg = getMessage(rule, messages, form, key);
      var error = getError(rule, validations, form, key, value, msg);
      if (error) { errors = errors.concat( [error]); }
    }
  });
  return errors;
}
function getAsyncErrors(validations, messages, form, key, value) {
  return new Promise(function ($return, $error) {
    var msg, error;
    msg = getMessage('customAsync', messages, form, key);
    return Promise.resolve(getError('customAsync', validations, form, key, value, msg)).then(function ($await_1) {
      try {
        error = $await_1;
        return $return(error);
      } catch ($boundEx) {
        return $error($boundEx);
      }
    }, $error);
  });
}

var services = /*#__PURE__*/Object.freeze({
  setMessages: setMessages,
  defaultState: defaultState,
  getData: getData,
  watchValidate: watchValidate,
  setProxy: setProxy,
  addTouchListener: addTouchListener,
  setListenersTouch: setListenersTouch,
  getSyncErrors: getSyncErrors,
  getAsyncErrors: getAsyncErrors
});

function setFormState(key, value, rules) {
  var obj;

  var filled = {
    isFilled: !!value
  };
  var dirted = {
    isDirty: !!value
  };
  return ( obj = {}, obj[key] = Object.assign({}, defaultState, dirted, filled, rules), obj);
}

function setFieldStates(form, key, value, rules) {
  var obj;

  this.validations = Object.assign({}, this.validations, ( obj = {}, obj[form] = Object.assign({}, this.validations[form], setFormState(key, value, rules)), obj));
}

function add(form, key, value, rules) {
  var vm = getContext.call(this);
  var addByDirective = typeof form === 'string';
  var addByUser = typeof form === 'object';

  if (addByDirective) {
    // prevents actions already taken
    if (!vm['validations'][form][key]) {
      setFieldStates.call(vm, form, key, value, rules);
      watchValidate.call(vm, form, key);
    }
  }

  if (addByUser) {
    Object.entries(form).forEach(function (ref) {
      var input = ref[0];
      var _rules = ref[1];

      // prevents actions already taken
      if (!vm['validations'][key][input]) {
        // key === formName
        setFieldStates.call(vm, key, input, value = '', _rules);
        watchValidate.call(vm, key, input);
      }
    });
  }

  setProxy.call(vm);
}

var context = {
  components: {} // proposal....
  // let components = {}
  // export default {
  //   components,
  //   set (component, vm) {
  //     components = {
  //       ...components,
  //       components: vm
  //     }
  //   },
  //   get (component) {
  //     return components[component]
  //   }
  // }

};

var defaultStates = ['isLoading', 'isChanged', 'isDirty', 'isFilled', 'isTouched', 'isValid'];

function setValue(state, value) {
  if (defaultStates.includes(state)) { return false; }
  if (state === 'errors') { return []; }
  return value;
}

function resetForm(fields) {
  return Object.entries(fields).reduce(function (accFields, ref) {
    var key = ref[0];
    var states = ref[1];

    accFields[key] = Object.entries(states).reduce(function (accStates, ref) {
      var state = ref[0];
      var value = ref[1];

      accStates[state] = setValue(state, value);
      return accStates;
    }, {});
    return accFields;
  }, {});
}

function reset (formToReset) {
  var vm = getContext.call(this);
  var isEnv = process.env.NODE_ENV === 'development';
  var hasForm = Object.keys(vm.validations).includes(formToReset);

  if (formToReset && !hasForm && isEnv) {
    console.warn(("it was not possible to reset the " + formToReset + " of a form that does not exist."));
    return;
  }

  if (!formToReset) { formToReset = Object.keys(vm.validations)[0]; }
  vm.validations = Object.entries(vm.validations).reduce(function (accForms, ref) {
    var form = ref[0];
    var fields = ref[1];

    form === formToReset ? accForms[form] = resetForm(fields) : accForms[form] = fields;
    return accForms;
  }, {});
  setProxy.call(vm);
  vm.$validator.validateOnBlur && setListenersTouch.call(vm);
}

function validate$1 (form) {
  var vm = getContext.call(this);
  if (!form) { form = Object.keys(vm.validations)[0]; }
  return new Promise(function (resolve) {
    var errors = [];
    Object.entries(vm.validations).filter(function (ref) {
      var formName = ref[0];

      return formName === form;
    }).forEach(function (ref) {
      var _ = ref[0];
      var states = ref[1];

      for (var key of Object.keys(states)) {
        validateField.call(vm, form, key, vm[form][key]);
        errors.push(vm.validations[form][key].isValid);
      }
    });
    var isFormValid = errors.every(function (error) { return error; });
    resolve(isFormValid);
  });
}

function setValidations(form, validations) {
  var this$1 = this;

  Object.keys(validations).forEach(function (input) { return watchValidate.call(this$1, form, input); });
}

function getFormStates(form, data, validation) {
  var obj;

  return ( obj = {}, obj[form] = Object.entries(validation).reduce(function (accForm, ref) {
      var input = ref[0];
      var rules = ref[1];

      var value = data[input];
      var filled = {
        isFilled: !!value
      };
      var dirted = {
        isDirty: !!value
      };
      accForm[input] = Object.assign({}, defaultState, dirted, filled, rules);
      return accForm;
    }, {}), obj);
}

function setFormStates(form, data, validation) {
  var newFormStates = getFormStates(form, data, validation);
  this.validations = Object.assign({}, this.validations, newFormStates);
}

function init() {
  var vm = getContext.call(this);
  var data = getData.call(vm);
  var validation = vm.$options.validation;

  var dataWithValidation = function (ref) {
    var key = ref[0];

    return !!(validation[key] && Object.keys(validation[key]).length);
  };

  Object.entries(data).filter(dataWithValidation).forEach(function (ref) {
    var form = ref[0];
    var data = ref[1];

    setFormStates.call(vm, form, data, validation[form]);
    setValidations.call(vm, form, validation[form]);
  });
  setProxy.call(vm);
}

var snapshots = {
  components: {}
};



var prototypes = /*#__PURE__*/Object.freeze({
  add: add,
  context: context,
  reset: reset,
  validate: validate$1,
  init: init,
  snapshots: snapshots
});

var messages = function (options) { return options && options.messages && {
  messages: options.messages
} || {}; };

var validateOnBlur = function (options) { return options && {
  validateOnBlur: options.validateOnBlur
}; };

var validator = {
  install: function install(Vue, options) {
    Object.defineProperty(Vue.prototype, '$validator', {
      get: function get() {
        return Object.assign({}, prototypes, services, messages(options), validateOnBlur(options));
      }

    });
  }

};

function getSnapshots() {
  var validation = this.$options.validation;
  var data = getData.call(this);
  return Object.entries(data).reduce(function (acc, ref) {
    var dataKey = ref[0];
    var dataValue = ref[1];

    Object.keys(validation).forEach(function (validationKey) { return validationKey === dataKey && (acc[dataKey] = dataValue); });
    return acc;
  }, {});
}
function setSnapshot() {
  var obj;

  this.$validator.snapshots.components = Object.assign({}, this.$validator.snapshots.components, ( obj = {}, obj[this._uid] = getSnapshots.call(this), obj));
}

var isCustomInput = function (vnode) { return vnode.data.model; };

var model = function (vnode) { return vnode.data.directives.reduce(function (acc, item) {
  if (item.name === 'model') { acc[item.name] = item; }
  return acc;
}, {}).model; };

function getFormValues(vnode, el) {
  var assign, assign$1;

  var form, key, value, inputElement; // check native or custom input

  if (isCustomInput(vnode)) {
    (assign = vnode.data.model.expression.split('.'), form = assign[0], key = assign[1]);
    value = vnode.data.model.value;
    inputElement = el && el.querySelector('input');
  } else {
    (assign$1 = model(vnode).expression.split('.'), form = assign$1[0], key = assign$1[1]);
    value = model(vnode).value;
    inputElement = el;
  }

  return {
    form: form,
    key: key,
    value: value,
    inputElement: inputElement
  };
}

var validator$1 = {
  validator: {
    bind: function bind(el, binding, vnode) {
      var vm = vnode.context;
      var ref = getFormValues(vnode);
      var form = ref.form; // if the form property does not exist in validations, set.

      if (!vm.validations[form]) {
        vm.$set.call(vnode, vm.validations, form, {});
      }
    },

    inserted: function inserted(el, ref, vnode) {
      var rules = ref.value;

      var vm = vnode.context;
      var ref$1 = getFormValues(vnode, el);
      var form = ref$1.form;
      var key = ref$1.key;
      var value = ref$1.value;
      var inputElement = ref$1.inputElement; // add validation states/rules

      vm.$validator.add(form, key, value, rules); // extra and optional flag to handle addTouchListener

      var addByDirective = true; // set validate onBlur

      vm.$validator.validateOnBlur && addTouchListener.call(vm, form, inputElement, addByDirective);
    }

  }
};

var formSetup = {
  created: function created() {
    var ref = this.$options;
    var validation = ref.validation;
    var messages = ref.messages;
    this.messages = messages || null;

    if (validation) {
      // set the component context values
      setContext.call(this); // set snapshot of the initial form

      setSnapshot.call(this); // overrides default messages based on global message options

      if (this.$validator.messages && this.messages && this.messages.length) {
        setMessages(this.messages, this.$validator.messages);
      } // create validation states/rules


      this.$validator.init(); // set validate onBlur

      this.$validator.validateOnBlur && setListenersTouch.call(this);
    }
  },

  directives: validator$1,

  data: function data() {
    return {
      validations: {},
      messages: {}
    };
  },

  methods: {
    $hasError: function $hasError(key, form) {
      if (this.validations && Object.keys(this.validations).length) {
        // in a single-form scenario, the scope is unique, and you do not have to explicitly name the form
        if (!form) { form = Object.keys(this.validations)[0]; }
        var input = this.validations[form] && this.validations[form][key];
        return input && input.isTouched && !input.isValid && input.errors[0];
      }

      return false;
    }

  }
};

exports.validator = validator;
exports.formSetup = formSetup;
